from pathlib import Path
import numpy as np
import matplotlib.pyplot as plt
from skimage.io import imread
from skimage.feature import match_template, peak_local_max
from dataclasses import dataclass
from argparse import ArgumentParser
from skimage.transform import rescale


@dataclass
class Image:
    name: str
    data: np.ndarray


parser = ArgumentParser(description='Image reader')
parser.add_argument('-t',
                    '--treshold',
                    type=float,
                    default=0.8,
                    help='Detection treshold for peak_local_max function.')
parser.add_argument('-r',
                    '--resize-alphabet',
                    dest='ratio',
                    type=float,
                    default=1.0,
                    help='Ratio for alphabet images resizing.')

args = parser.parse_args()


def load_images_from_dir(path: str, rescaling: bool = False) -> list[Image]:
    files: list[Image] = []
    # load images as grayscale to cut any additional unecessary
    # dimensions (like alpha channel) and for easier detecion
    for entry in Path(path).iterdir():
        if entry.is_file():
            image_data: np.ndarray = imread(entry.resolve(), as_gray=True)
            # if ratio is set, then rescale loaded image,
            # if rescaling also enabled in function call
            ratio: float = args.ratio
            if ratio != 1.0 and rescaling:
                image_data = rescale(image_data, ratio, anti_aliasing=True)
            files.append(
                Image(name=entry.name.split(".")[0],
                      data=image_data))
    return files


letters: list[Image] = load_images_from_dir("alphabet", rescaling=True)
text_images: list[Image] = load_images_from_dir("texts")

for text in text_images:
    # (x, y) -> {letter}
    letters_positons: dict[tuple[int, int], str] = {}
    for letter in letters:
        result: np.ndarray = match_template(text.data, letter.data)
        # peaks are tuples with coordinates of left top corner
        # of found pattern in form (y, x)
        peaks: np.ndarray = peak_local_max(result,
                                           threshold_abs=args.treshold,
                                           min_distance=10)
        if len(peaks) == 0:
            continue

        fig = plt.figure(figsize=(15, 7))
        ax1 = plt.subplot(1, 2, 1)
        ax2 = plt.subplot(1, 2, 2)

        ax1.imshow(letter.data, cmap="gray")
        ax1.set_axis_off()
        ax1.set_title(letter.name)

        ax2.imshow(text.data, cmap="gray")
        ax2.set_axis_off()
        ax2.set_title(text.name)

        for y, x in peaks:
            # save letters
            letters_positons[(x, y)] = letter.name

            # highlight matched region
            h, w = letter.data.shape
            rect = plt.Rectangle((x, y), w, h, edgecolor='r', facecolor='none')
            ax2.add_patch(rect)

        plt.show()
    # sort according to x positon
    keys: list[tuple[int, int]] = [*letters_positons.keys()]
    keys.sort(key=lambda x: x[0])
    print(f"In image '{text.name}' found word: "
          f"{''.join([letters_positons[x] for x in keys])}")
